@@ -17,16 +17,16 @@
 
 use std::time::Duration;
 
+use jni::JNIEnv;
 use jni::objects::JClass;
 use jni::sys::jboolean;
 use jni::sys::jfloat;
 use jni::sys::jlong;
-use jni::JNIEnv;
+use opendal::Operator;
 use opendal::layers::ConcurrentLimitLayer;
 use opendal::layers::RetryLayer;
-use opendal::Operator;
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "system" fn Java_org_apache_opendal_layer_RetryLayer_doLayer(
     _: JNIEnv,
     _: JClass,
@@ -51,7 +51,7 @@ pub extern "system" fn Java_org_apache_opendal_layer_RetryLayer_doLayer(
     Box::into_raw(Box::new(op.clone().layer(retry))) as jlong
 }
 
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub extern "system" fn Java_org_apache_opendal_layer_ConcurrentLimitLayer_doLayer(
     _: JNIEnv,
     _: JClass,
