@@ -17,17 +17,17 @@
 
 use std::collections::HashMap;
 
+use jni::JNIEnv;
 use jni::objects::JObject;
 use jni::objects::JValue;
 use jni::sys::jboolean;
 use jni::sys::jint;
 use jni::sys::jlong;
-use jni::JNIEnv;
-use opendal::raw::PresignedRequest;
 use opendal::Entry;
 use opendal::EntryMode;
 use opendal::Metadata;
 use opendal::OperatorInfo;
+use opendal::raw::PresignedRequest;
 use opendal::{Capability, Error, ErrorKind};
 
 mod async_operator;
@@ -156,8 +156,8 @@ fn make_metadata<'a>(env: &mut JNIEnv<'a>, metadata: Metadata) -> Result<JObject
                     "ofEpochSecond",
                     "(JJ)Ljava/time/Instant;",
                     &[
-                        JValue::Long(v.timestamp()),
-                        JValue::Long(v.timestamp_subsec_nanos() as jlong),
+                        JValue::Long(v.into_inner().as_second()),
+                        JValue::Long(v.into_inner().subsec_nanosecond() as jlong),
                     ],
                 )?
                 .l()?)
@@ -219,7 +219,7 @@ fn make_write_options<'a>(
                 ErrorKind::Unexpected,
                 format!("Concurrent must be positive, instead got: {v}"),
             )
-            .into())
+            .into());
         }
     };
     Ok(opendal::options::WriteOptions {
@@ -254,12 +254,12 @@ fn make_stat_options(env: &mut JNIEnv, options: &JObject) -> Result<opendal::opt
     Ok(opendal::options::StatOptions {
         if_match: convert::read_string_field(env, options, "ifMatch")?,
         if_none_match: convert::read_string_field(env, options, "ifNoneMatch")?,
-        if_modified_since: convert::read_instant_field_to_date_time(
+        if_modified_since: convert::read_instant_field_to_timestamp(
             env,
             options,
             "ifModifiedSince",
         )?,
-        if_unmodified_since: convert::read_instant_field_to_date_time(
+        if_unmodified_since: convert::read_instant_field_to_timestamp(
             env,
             options,
             "ifUnmodifiedSince",
