@@ -15,11 +15,11 @@
 // specific language governing permissions and limitations
 // under the License.
 
+use jni::JNIEnv;
 use jni::objects::JByteArray;
 use jni::objects::JClass;
 use jni::objects::JString;
 use jni::sys::jlong;
-use jni::JNIEnv;
 use opendal::blocking;
 
 use crate::convert::jstring_to_string;
@@ -27,14 +27,15 @@ use crate::convert::jstring_to_string;
 /// # Safety
 ///
 /// This function should not be called before the Operator is ready.
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub unsafe extern "system" fn Java_org_apache_opendal_OperatorOutputStream_constructWriter(
     mut env: JNIEnv,
     _: JClass,
     op: *mut blocking::Operator,
     path: JString,
 ) -> jlong {
-    intern_construct_write(&mut env, &mut *op, path).unwrap_or_else(|e| {
+    let op_ref = unsafe { &mut *op };
+    intern_construct_write(&mut env, op_ref, path).unwrap_or_else(|e| {
         e.throw(&mut env);
         0
     })
@@ -53,13 +54,13 @@ fn intern_construct_write(
 /// # Safety
 ///
 /// This function should not be called before the Operator is ready.
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub unsafe extern "system" fn Java_org_apache_opendal_OperatorOutputStream_disposeWriter(
     mut env: JNIEnv,
     _: JClass,
     writer: *mut blocking::Writer,
 ) {
-    let mut writer = Box::from_raw(writer);
+    let mut writer = unsafe { Box::from_raw(writer) };
     intern_dispose_write(&mut writer).unwrap_or_else(|e| {
         e.throw(&mut env);
     })
@@ -73,14 +74,15 @@ fn intern_dispose_write(writer: &mut blocking::Writer) -> crate::Result<()> {
 /// # Safety
 ///
 /// This function should not be called before the Operator is ready.
-#[no_mangle]
+#[unsafe(no_mangle)]
 pub unsafe extern "system" fn Java_org_apache_opendal_OperatorOutputStream_writeBytes(
     mut env: JNIEnv,
     _: JClass,
     writer: *mut blocking::Writer,
     content: JByteArray,
 ) {
-    intern_write_bytes(&mut env, &mut *writer, content).unwrap_or_else(|e| {
+    let writer_ref = unsafe { &mut *writer };
+    intern_write_bytes(&mut env, writer_ref, content).unwrap_or_else(|e| {
         e.throw(&mut env);
     })
 }
